// -*-c++-*-

#ifndef ZONE_INFO_H
#define ZONE_INFO_H

#include <string>
#include <rdbxx/RDB.h>

#include <utils.h>

namespace Scatter {

    using namespace std;

    struct ZoneInfo {

	string mirror;
	string zoneid;
	string scatalg;

	bool in_plane;
	bool out_of_plane;

	string scatdb_in_plane;
	string scatdb_out_of_plane;

	double scale_in_plane;
	double scale_out_of_plane;

	double z0;
	double zmin;
	double zmax;

	string filename;

	bool exists ( RDB& db, const char* colname ) {

	    try{
		db.getColumn( colname );
		return true;

	    }
	    catch ( ... ) {

		return false;
	    }

	}

	ZoneInfo( RDB& db, const string& filename, double z0, bool in_plane, bool out_of_plane ) :
	    in_plane( in_plane ),
	    out_of_plane( out_of_plane ),
	    scale_in_plane(1),
	    scale_out_of_plane(1),
	    z0( z0 ),
	    filename( filename )
	{

	    db.mapData( "mirror"	, &mirror );
	    db.mapData( "zone"	, &zoneid );

	    // for now, zones are cylindrical patches
	    db.mapData( "zmin"	, &zmin );
	    db.mapData( "zmax"	, &zmax );

	    // handle one or two scatter dbs.  "scat_file" or "scatdb_in_plane" is  the in-plane table
	    // "scatdb_out_of_plane" is the out_of_plane db

	    if ( exists( db, "scat_file" ) ) {

		db.mapData( "scat_file", &scatdb_in_plane );

		if ( exists( db, "scatdb_in_plane" ) )
		    throw( Exception( filename + ": cannot have both scatfile and scatdb_in_plane columns" ) );
	    }

	    else if ( exists( db, "scatdb_in_plane" ) ) {

		db.mapData( "scatdb_in_plane", &scatdb_in_plane );
	    }

	    else {

		throw( Exception( filename + ": missing column for in-plane scatter db" ) );

	    }

	    if ( exists( db, "scatdb_out_of_plane" ) )
		db.mapData( "scatdb_out_of_plane", &scatdb_out_of_plane );

	    if ( exists( db, "scale_in_plane" ) )
		db.mapData( "scale_in_plane", &scale_in_plane );

	    if ( exists( db, "scale_out_of_plane" ) )
		db.mapData( "scale_out_of_plane", &scale_out_of_plane );

	}

	void interp_env() {

	    scatdb_in_plane = string_interp( scatdb_in_plane );
	    scatdb_out_of_plane = string_interp( scatdb_out_of_plane );

	}

    };

}
#endif // ! ZONE_INFO_H
